"use strict";

function _call(body, then, direct) {
  if (direct) {
    return then ? then(body()) : body();
  }

  try {
    var result = Promise.resolve(body());
    return then ? result.then(then) : result;
  } catch (e) {
    return Promise.reject(e);
  }
}

function _async(f) {
  return function () {
    for (var args = [], i = 0; i < arguments.length; i++) {
      args[i] = arguments[i];
    }

    try {
      return Promise.resolve(f.apply(this, args));
    } catch (e) {
      return Promise.reject(e);
    }
  };
}

Object.defineProperty(exports, "__esModule", {
  value: true
});

var tslib_1 = require("tslib");

var taro_1 = require("@tarojs/taro");

var ahooks_1 = require("@taro-hooks/ahooks");

var shared_1 = require("@taro-hooks/shared");

var taro_hooks_1 = require("taro-hooks");

var limit_1 = tslib_1.__importDefault(require("../utils/limit"));

var isOnline_1 = tslib_1.__importDefault(require("../utils/isOnline"));

var listeners = [];

function subscribe(listener) {
  listeners.push(listener);
  return function unsubscribe() {
    var index = listeners.indexOf(listener);
    listeners.splice(index, 1);
  };
}

var useRefreshOnWindowFocusPlugin = function useRefreshOnWindowFocusPlugin(fetchInstance, _ref) {
  var refreshOnWindowFocus = _ref.refreshOnWindowFocus,
      _ref$focusTimespan = _ref.focusTimespan,
      focusTimespan = _ref$focusTimespan === void 0 ? 5000 : _ref$focusTimespan;
  var unsubscribeRef = (0, taro_1.useTaroRef)();
  var visible = (0, taro_hooks_1.useVisible)();

  var stopSubscribe = function stopSubscribe() {
    unsubscribeRef.current == null ? void 0 : unsubscribeRef.current();
  };

  var subscribeFocus = _async(function () {
    return _call((0, isOnline_1["default"]), function (online) {
      if (!(0, shared_1.escapeState)(visible) || !online) return;

      for (var i = 0; i < listeners.length; i++) {
        var listener = listeners[i];
        listener();
      }
    });
  });

  (0, taro_1.useDidShow)(function () {
    subscribeFocus();
  });
  (0, taro_1.useDidHide)(function () {
    subscribeFocus();
  });
  (0, taro_1.useTaroEffect)(function () {
    if (refreshOnWindowFocus) {
      var limitRefresh = (0, limit_1["default"])(fetchInstance.refresh.bind(fetchInstance), focusTimespan);
      unsubscribeRef.current = subscribe(function () {
        limitRefresh();
      });
    }

    return function () {
      stopSubscribe();
    };
  }, [refreshOnWindowFocus, focusTimespan, visible]);
  (0, ahooks_1.useUnmount)(function () {
    stopSubscribe();
  });
  return {};
};

exports["default"] = useRefreshOnWindowFocusPlugin;